/*
    SPDX-FileCopyrightText: 2019 Aleix Pol i Gonzalez <aleixpol@blue-systems.com>

    SPDX-License-Identifier: LGPL-2.1-or-later
*/

#include "abstracteditablecontact.h"

using namespace KPeople;

AbstractEditableContact::AbstractEditableContact()
{
}

AbstractEditableContact::~AbstractEditableContact() = default;
